//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "main.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma link "wclSerialDevices"
#pragma resource "*.dfm"
TfmMain *fmMain;
//---------------------------------------------------------------------------
__fastcall TfmMain::TfmMain(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormCreate(TObject *Sender)
{
  EnumComPorts();

  ClearConfig();
  ClearTiemouts();
  ClearBuffers();

  edWriteTimeout->Text = IntToStr((int)wclSerialClient->WriteTimeout);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::FormDestroy(TObject *Sender)
{
  wclSerialClient->Disconnect();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearClick(TObject *Sender)
{
  lbEvents->Clear();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ReadConfiguration()
{
  TwclSerialConfig Config;
  int Res = wclSerialClient->GetConfig(Config);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Read configuration error: 0x" + IntToHex(Res, 8));

  else
  {
    edBaudRate->Text = IntToStr((int)Config.BaudRate);
    edXonLim->Text = IntToStr((int)Config.XonLim);
    edXoffLim->Text = IntToStr((int)Config.XoffLim);
    edXonChar->Text = IntToStr((int)Config.XonChar);
    edXoffChar->Text = IntToStr((int)Config.XoffChar);
    edErrorChar->Text = IntToStr((int)Config.ErrorChar);
    edEofChar->Text = IntToStr((int)Config.EofChar);
    edEvtChar->Text = IntToStr((int)Config.EvtChar);

    cbParityCheck->Checked = Config.ParityCheck;
    cbOutxCtsFlow->Checked = Config.OutxCtsFlow;
    cbOutxDsrFlow->Checked = Config.OutxDsrFlow;
    cbDsrSensitivity->Checked = Config.DsrSensitivity;
    cbTXContinueOnXoff->Checked = Config.TXContinueOnXoff;
    cbOutX->Checked = Config.OutX;
    cbInX->Checked = Config.InX;
    cbErrorCharReplace->Checked = Config.ErrorCharReplace;
    cbNullStrip->Checked = Config.NullStrip;
    cbAbortOnError->Checked = Config.AbortOnError;

    cbRtsControl->ItemIndex = (int)Config.RtsControl;
    cbDtrControl->ItemIndex = (int)Config.DtrControl;
    cbByteSize->ItemIndex = Config.ByteSize - 4;
    cbParity->ItemIndex = (int)Config.Parity;
    cbStopBits->ItemIndex = (int)Config.StopBits;
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ReadTiemouts()
{
  TwclSerialTimeouts Times;
  int Res = wclSerialClient->GetTimeouts(Times);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Get timeouts error: 0x" + IntToHex(Res, 8));

  else
  {
    edReadInterval->Text = IntToStr((int)Times.ReadInterval);
    edReadMultiplier->Text = IntToStr((int)Times.ReadMultiplier);
    edReadConstant->Text = IntToStr((int)Times.ReadConstant);
    edWriteMultiplier->Text = IntToStr((int)Times.WriteMultiplier);
    edWriteConstant->Text = IntToStr((int)Times.WriteConstant);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btEnumClick(TObject *Sender)
{
  EnumComPorts();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::EnumComPorts()
{
  cbPorts->Clear();

  TwclSerialDevices Ports;
  int Res = wclSerialMonitor->EnumDevices(Ports);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Error enumerating COM ports: 0x" + IntToHex(Res, 8));

  else
  {
    for (int i = 0; i < Ports.Length; i++)
      cbPorts->Items->Add(Ports[i].DeviceName);

    if (cbPorts->Items->Count > 0)
      cbPorts->ItemIndex = 0;
    else
      cbPorts->ItemIndex = -1;
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btConnectClick(TObject *Sender)
{
  if (cbPorts->ItemIndex == -1)
    MessageDlg("Select COM port", mtWarning, TMsgDlgButtons() << mbOK, 0);

  else
  {
    wclSerialClient->DeviceName = cbPorts->Items->Strings[cbPorts->ItemIndex];

    int Res = wclSerialClient->Connect();
    if (Res != WCL_E_SUCCESS)
      MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
  }
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btDisconnectClick(TObject *Sender)
{
  int Res = wclSerialClient->Disconnect();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ClearConfig()
{
  edBaudRate->Text = "";
  edXonLim->Text = "";
  edXoffLim->Text = "";
  edXonChar->Text = "";
  edXoffChar->Text = "";
  edErrorChar->Text = "";
  edEofChar->Text = "";
  edEvtChar->Text = "";

  cbParityCheck->Checked = false;
  cbOutxCtsFlow->Checked = false;
  cbOutxDsrFlow->Checked = false;
  cbDsrSensitivity->Checked = false;
  cbTXContinueOnXoff->Checked = false;
  cbOutX->Checked = false;
  cbInX->Checked = false;
  cbErrorCharReplace->Checked = false;
  cbNullStrip->Checked = false;
  cbAbortOnError->Checked = false;

  cbRtsControl->ItemIndex = -1;
  cbDtrControl->ItemIndex = -1;
  cbByteSize->ItemIndex = -1;
  cbParity->ItemIndex = -1;
  cbStopBits->ItemIndex = -1;
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btSetConfigClick(TObject *Sender)
{
  TwclSerialConfig Config;
  Config.BaudRate = StrToInt64(edBaudRate->Text);
  Config.XonLim = StrToInt(edXonLim->Text);
  Config.XoffLim = StrToInt(edXoffLim->Text);
  Config.XonChar = Char(StrToInt(edXonChar->Text));
  Config.XoffChar = Char(StrToInt(edXoffChar->Text));
  Config.ErrorChar = Char(StrToInt(edErrorChar->Text));
  Config.EofChar = Char(StrToInt(edEofChar->Text));
  Config.EvtChar = Char(StrToInt(edEvtChar->Text));

  Config.ParityCheck = cbParityCheck->Checked;
  Config.OutxCtsFlow = cbOutxCtsFlow->Checked;
  Config.OutxDsrFlow = cbOutxDsrFlow->Checked;
  Config.DsrSensitivity = cbDsrSensitivity->Checked;
  Config.TXContinueOnXoff = cbTXContinueOnXoff->Checked;
  Config.OutX = cbOutX->Checked;
  Config.InX = cbInX->Checked;
  Config.ErrorCharReplace = cbErrorCharReplace->Checked;
  Config.NullStrip = cbNullStrip->Checked;
  Config.AbortOnError = cbAbortOnError->Checked;

  Config.RtsControl = (TwclSerialRtsControl)cbRtsControl->ItemIndex;
  Config.DtrControl = (TwclSerialDtrControl)cbDtrControl->ItemIndex;
  Config.ByteSize = cbByteSize->ItemIndex + 4;
  Config.Parity = (TwclSerialParity)cbParity->ItemIndex;
  Config.StopBits = (TwclSerialStopBits)cbStopBits->ItemIndex;

  int Res = wclSerialClient->SetConfig(Config);
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btGetConfigClick(TObject *Sender)
{
  ReadConfiguration();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ClearBuffers()
{
  edReadBufferSize->Text = "";
  edWriteBufferSize->Text = "";
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ClearTiemouts()
{
  edReadInterval->Text = "";
  edReadMultiplier->Text = "";
  edReadConstant->Text = "";
  edWriteMultiplier->Text = "";
  edWriteConstant->Text = "";
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::ReadBuffers()
{
  unsigned int Size;
  int Res = wclSerialClient->GetReadBufferSize(Size);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Get read buffer size error: 0x" + IntToHex(Res, 8));
  else
    edReadBufferSize->Text = IntToStr((int)Size);

  Res = wclSerialClient->GetWriteBufferSize(Size);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Get write buffer size error: 0x" + IntToHex(Res, 8));
  else
    edWriteBufferSize->Text = IntToStr((int)Size);
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btGetBuffersClick(TObject *Sender)
{
  ReadBuffers();        
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btSetBuffersClick(TObject *Sender)
{
  int Res = wclSerialClient->SetReadBufferSize(StrToInt64(edReadBufferSize->Text));
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Set read buffer size error: 0x" + IntToHex(Res, 8));

  Res = wclSerialClient->SetWriteBufferSize(StrToInt64(edWriteBufferSize->Text));
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Set write buffer size error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btGetTimeoutsClick(TObject *Sender)
{
  ReadTiemouts();
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btSetTimeoutsClick(TObject *Sender)
{
  TwclSerialTimeouts Times;
  Times.ReadInterval = StrToInt64(edReadInterval->Text);
  Times.ReadMultiplier = StrToInt64(edReadMultiplier->Text);
  Times.ReadConstant = StrToInt64(edReadConstant->Text);
  Times.WriteMultiplier = StrToInt64(edWriteMultiplier->Text);
  Times.WriteConstant = StrToInt64(edWriteConstant->Text);
  int Res = wclSerialClient->SetTimeouts(Times);
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Set timeouts error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------
void __fastcall TfmMain::btClearCommBreakClick(TObject *Sender)
{
  int Res = wclSerialClient->ClearCommBreak();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSetCommBreakClick(TObject *Sender)
{
  int Res = wclSerialClient->SetCommBreak();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btFlushBuffersClick(TObject *Sender)
{
  int Res = wclSerialClient->FlushBuffers();
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btFuncClick(TObject *Sender)
{
  int Res = wclSerialClient->EscapeCommFunction((TwclSerialEscapeFunction)cbFunc->ItemIndex);
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btPurgeClick(TObject *Sender)
{
  TwclSerialPurgeFlags Flags;
  Flags.Clear();
  if (cbpurgeRxAbort->Checked)
    Flags << purgeRxAbort;
  if (cbpurgeRxClear->Checked)
    Flags << purgeRxClear;
  if (cbpurgeTxAbort->Checked)
    Flags << purgeTxAbort;
  if (cbpurgeTxClear->Checked)
    Flags << purgeTxClear;

  int Res = wclSerialClient->PurgeComm(Flags);
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btTransmitClick(TObject *Sender)
{
  int Res = wclSerialClient->TransmitCommChar(StrToInt(edChar->Text));
  if (Res != WCL_E_SUCCESS)
    MessageDlg("Error: 0x" + IntToHex(Res, 8), mtError, TMsgDlgButtons() << mbOK, 0);        
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSendClick(TObject *Sender)
{
  AnsiString Str = AnsiString(edText->Text);
  switch (cbLineFeed->ItemIndex)
  {
    case 1:
      Str += '\r';
      break;
    case 2:
      Str += '\n';
      break;
    case 3:
      Str += "\r\n";
      break;
  }
  unsigned int Written;
  int Res = wclSerialClient->Write(Str.c_str(), Str.Length(), Written);
  lbEvents->Items->Add("Sent: " + IntToStr((int)Written) + " bytes from " +
    IntToStr(Str.Length()));
  if (Res != WCL_E_SUCCESS)
    lbEvents->Items->Add("Write error: 0x" + IntToHex(Res, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientConnect(TObject *Sender,
      const int Error)
{
  if (Error == WCL_E_SUCCESS)
  {
    lbEvents->Items->Add("Connected to Serial Device: " +
      wclSerialClient->DeviceName);

    ReadConfiguration();
    ReadTiemouts();
    ReadBuffers();
  }
  else
    lbEvents->Items->Add("Connect error: 0x" + IntToHex(Error, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientData(TObject *Sender,
      const Pointer Data, const DWORD Size)
{
  if (Size > 0)
  {
    AnsiString Str((char*)Data, Size);
    lbEvents->Items->Add("Received: " + Str);
  }
  else
    lbEvents->Items->Add("Empty data received");
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientDisconnect(TObject *Sender,
      const int Reason)
{
  lbEvents->Items->Add("Disconnected: 0x" + IntToHex(Reason, 8));

  ClearConfig();
  ClearTiemouts();
  ClearBuffers();
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientError(TObject *Sender,
      const TwclSerialErrors Errors,
      const TwclSerialCommunicationStates States)
{
  String Str = "";
  if (Errors.Contains(erBreak))
    Str = Str + "erBreak ";
  if (Errors.Contains(erFrame))
    Str = Str + "erFrame ";
  if (Errors.Contains(erOverrun))
    Str = Str + "erOverrun ";
  if (Errors.Contains(erRxOver))
    Str = Str + "erRxOver ";
  if (Errors.Contains(erRxParity))
    Str = Str + "erRxParity ";
  lbEvents->Items->Add("Error: " + Str);

  Str = "";
  if (States.Contains(csCtsHold))
    Str = Str + "csCtsHold ";
  if (States.Contains(csDsrHold))
    Str = Str + "csDsrHold ";
  if (States.Contains(csRlsdHold))
    Str = Str + "csRlsdHold ";
  if (States.Contains(csXoffHold))
    Str = Str + "csXoffHold ";
  if (States.Contains(csXoffSent))
    Str = Str + "csXoffSent ";
  if (States.Contains(csEof))
    Str = Str + "csEof ";
  if (States.Contains(csTxim))
    Str = Str + "csTxim ";
  lbEvents->Items->Add("States: " + Str);
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientReadError(TObject *Sender,
      const int Error)
{
  lbEvents->Items->Add("Read error: 0x" + IntToHex(Error, 8));
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::wclSerialClientEvents(TObject *Sender,
      const TwclSerialEvents Events)
{
  String Str = "";
  if (Events.Contains(evBreak))
    Str = Str + "evBreak ";
  if (Events.Contains(evCts))
    Str = Str + "evCts ";
  if (Events.Contains(evDsr))
    Str = Str + "evDsr ";
  if (Events.Contains(evRing))
    Str = Str + "evRing ";
  if (Events.Contains(evRlsd))
    Str = Str + "evRlsd ";
  if (Events.Contains(evChar))
    Str = Str + "evChar ";
  lbEvents->Items->Add("Event: " + Str);

  if (!Events.Empty())
  {
    TwclModemStatuses Status;
    int Res = wclSerialClient->GetModemStatus(Status);
    if (Res != WCL_E_SUCCESS)
      lbEvents->Items->Add("GetModemStatus error: 0x" + IntToHex(Res, 8));
    else
    {
      Str = "";
      if (Status.Contains(msCtsOn))
        Str = Str + "msCtsOn ";
      if (Status.Contains(msDsrOn))
        Str = Str + "msDsrOn ";
      if (Status.Contains(msRingOn))
        Str = Str + "msRingOn ";
      if (Status.Contains(msDsrOn))
        Str = Str + "msDsrOn ";
      if (Status.Contains(msRlsdOn))
        Str = Str + "msRlsdOn ";
      if (Str != "")
        lbEvents->Items->Add("Modem status: " + Str);
    }
  }
}
//---------------------------------------------------------------------------

void __fastcall TfmMain::btSetWriteTimeoutClick(TObject *Sender)
{
  wclSerialClient->WriteTimeout = StrToInt(edWriteTimeout->Text);        
}
//---------------------------------------------------------------------------

